/****************************************************************************\
 *           Copyright (C) 2012 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
This example shows SPI master mode operations:
1. SPI loopback
2. read/write SPI EEPROM
 ****************************************************************************
 * Created by :                                                             *
 *            Brighton Feng                                                 *
 *            Texas Instruments                                             * 
 *            Feb 25, 2012                                                 *
***************************************************************************/
#include <c6x.h>
#include <stdio.h>
#include <string.h>
#include <csl_bootcfgAux.h>
#include "Keystone_common.h"
#include "Keystone_SPI_init_drv.h"

#define SPI_LOOPBACK_TEST 		1
#define SPI_EEPROM_TEST 		1

#if SPI_EEPROM_TEST
#include "SPI_EEPROM_Test.h"
#include "SPI_EEPROM_drv.h"
#endif

#if SPI_LOOPBACK_TEST
#define SPI_LOOP_TEST_BUF_SIZE 	(64*1024)
Uint16 spiRxBuf[SPI_LOOP_TEST_BUF_SIZE/2];
Uint16 spiTxBuf[SPI_LOOP_TEST_BUF_SIZE/2];
#endif

SPI_Data_Format dataFormat[2];
SPI_Config spiCfg;

void SPI_Init()
{
#if SPI_EEPROM_TEST
	/*data format for EEPROM test*/
	dataFormat[0].delayBetweenTrans_ns=0;   
	dataFormat[0].ShifDirection       =SPI_MSB_SHIFT_FIRST;
	dataFormat[0].disable_CS_timing   =0;
	dataFormat[0].clockPolarity       =SPI_CLOCK_LOW_INACTIVE;
	dataFormat[0].clockPhase          =1;
	dataFormat[0].wordLength          =8;
	spiCfg.dataFormat[0]= &dataFormat[0];
#endif

#if SPI_LOOPBACK_TEST
	/*data format for loopback test*/
	dataFormat[1].delayBetweenTrans_ns=0;   
	dataFormat[1].ShifDirection       =SPI_MSB_SHIFT_FIRST;
	dataFormat[1].disable_CS_timing   =1;
	dataFormat[1].clockPolarity       =SPI_CLOCK_LOW_INACTIVE;
	dataFormat[1].clockPhase          =0;
	dataFormat[1].clockSpeedKHz       =66000;
	dataFormat[1].wordLength          =16;
	spiCfg.dataFormat[1]= &dataFormat[1];
#endif

	spiCfg.DMA_requestEnable= FALSE;
	spiCfg.interruptCfg     = NULL;
	spiCfg.number_SPI_pins  = 4;
	spiCfg.CS_polarity      =SPI_CS_LOW_ACTIVE;

	KeyStone_SPI_init(&spiCfg);
}

int SPI_loopback_test_one_time(Uint16 *txBuf, Uint16 *rxBuf, Uint32 uiByteCount)
{
	int i, iByteSuccess;
	
	iByteSuccess= KeyStone_SPI_TxRx((Uint8 *) txBuf, 0, uiByteCount, 
		(Uint8 *) rxBuf, 0, uiByteCount, SPI_CS_NO_LAST_HOLD, FALSE, 1, 1);

	if(iByteSuccess!=uiByteCount)
		return iByteSuccess;
		
	for(i=0; i<uiByteCount/2; i++)
	{
		if(txBuf[i]!=rxBuf[i])
		{
			printf("SPI test failed at word %d: TX 0x%x, RX 0x%x\n",
				i, txBuf[i], rxBuf[i]);
			break;
		}
	}

	return uiByteCount;
}
void SPI_loopback_test(Uint16 *txBuf, Uint16 *rxBuf, Uint32 uiByteCount)
{
	int i;
	
	for(i=0; i<uiByteCount/2; i++)
	{
		txBuf[i]= 0;
		rxBuf[i]= 0xFFFF;
	}
	if(SPI_loopback_test_one_time(txBuf, rxBuf, uiByteCount)==uiByteCount)
		puts("SPI loopback test passed with data pattern 0");
	
	for(i=0; i<uiByteCount/2; i++)
	{
		txBuf[i]= 0xFFFF;
		rxBuf[i]= 0;
	}
	if(SPI_loopback_test_one_time(txBuf, rxBuf, uiByteCount)==uiByteCount)
		puts("SPI loopback test passed with data pattern 0xFFFF");
	
	for(i=0; i<uiByteCount/2; i++)
	{
		txBuf[i]= (Uint16)&rxBuf[i];
		rxBuf[i]= 0;
	}
	if(SPI_loopback_test_one_time(txBuf, rxBuf, uiByteCount)==uiByteCount)
		puts("SPI loopback test passed with data pattern = data address");
	
}

main()
{
	int i;
#if SPI_EEPROM_TEST
	int iEEPROM_size_KB, iEEPROM_addres_width;
#endif
	
	TDSP_Type dspType;

    CACHE_setL1PSize(CACHE_L1_32KCACHE);
    CACHE_setL1DSize(CACHE_L1_32KCACHE);
    CACHE_setL2Size(CACHE_0KCACHE);

    IER= 0;
	Exception_cfg();	//for debug

	CSL_BootCfgUnlockKicker();

	/*clear configuration structure to default values*/
	memset(&spiCfg, 0, sizeof(spiCfg));
	
	dspType= Get_dsp_type();
	if(TCI6608==dspType)
	{
		//DSP core speed
		//KeyStone_main_PLL_init (20, 1); 	//for 50MHz input clock
		KeyStone_main_PLL_init (10, 1); 	//for 100MHz input clock

		//On Shannon, SPI belong to clock domain 3 which need be enabled
		Keystone_enable_PSC_module(CSL_PSC_PD_ALWAYSON, 3);
#if SPI_EEPROM_TEST
		/*N25Q128 EEPROM test on Shannon EVM*/
		dataFormat[0].clockSpeedKHz =66000;
		iEEPROM_size_KB= 1*1024;
		iEEPROM_addres_width = 24;
		uiSPI_EEPROM_page_size= 256;
		needEraseBeforeWrite= TRUE;
#endif
	}
	else if(TCI6616==dspType||TCI6618==dspType)
	{
		//DSP core speed
		KeyStone_main_PLL_init (236, 29); 	//for 122.88MHz input clock

#if SPI_EEPROM_TEST
		/*AT25640B EEPROM test on Nyquist EVM*/
		dataFormat[0].clockSpeedKHz =5000;
		iEEPROM_size_KB= 8;
		iEEPROM_addres_width = 16;
		uiSPI_EEPROM_page_size= 32;
		needEraseBeforeWrite= FALSE;
#endif
	}
	else
	{
		puts("Unknown DSP type!");
		return 1;
	}

	SPI_Init();

#if SPI_LOOPBACK_TEST
	puts("SPI internal loopback test start...");
	spiCfg.loopBack = TRUE;
	SPI_Init();
	SPI_loopback_test(spiTxBuf, spiRxBuf, SPI_LOOP_TEST_BUF_SIZE);
#endif
	
#if SPI_EEPROM_TEST
	puts("SPI EEPROM test start...");
	spiCfg.loopBack = FALSE;
	SPI_Init();
	if(iEEPROM_size_KB<=256)
		SPI_EEPROM_Test(0, 0, iEEPROM_size_KB*1024, iEEPROM_addres_width);
	else
	{
		/*spilt large space into small sections to show the progress*/
		for(i=0; i<iEEPROM_size_KB; i+= 256)
			SPI_EEPROM_Test(0, i*1024, (i+256)*1024, iEEPROM_addres_width);
	}
#endif
	
	puts("SPI test complete.");
	CACHE_invAllL1p(CACHE_WAIT);
	CACHE_wbInvAllL1d(CACHE_WAIT);
	InvalidPrefetchBuffer();
}


